﻿using Hotline.Orders;
using Hotline.Orders.Notifications;
using Hotline.Settings;
using Hotline.Share.Dtos;
using Hotline.Share.Enums.Order;
using MediatR;
using XF.Domain.Repository;


namespace Hotline.Application.Handlers.Order
{
    public class AddVisitNotifyHandler : INotificationHandler<AddVisitNotify>
    {


        private readonly IRepository<OrderVisit> _orderVisitRepository;
        private readonly IRepository<OrderVisitDetail> _orderVisitedDetailRepository;
        private readonly IRepository<SystemDicData> _systemDicDataRepository;
        private readonly IOrderRepository _orderRepository;

        public AddVisitNotifyHandler(IRepository<OrderVisit> orderVisitRepository, IRepository<OrderVisitDetail> orderVisitedDetailRepository,IRepository<SystemDicData> systemDicDataRepository,IOrderRepository orderRepository)
        {
            _orderVisitRepository = orderVisitRepository;
            _orderVisitedDetailRepository = orderVisitedDetailRepository;
            _systemDicDataRepository = systemDicDataRepository;
            _orderRepository = orderRepository;
        }


        public async Task Handle(AddVisitNotify notification, CancellationToken cancellationToken)
        {
            var orderVisit = await _orderVisitRepository.Queryable()
                .Includes(x=>x.Order)
                .Includes(x=>x.OrderVisitDetails)
                .Where(x => x.Order.ProvinceNo == notification.OrderVisited.ProvinceNo && 
             x.VisitState != Share.Enums.Order.EVisitState.None).FirstAsync(cancellationToken);

            if (orderVisit != null)
            {
                //处理评价结果
                var processingResult = notification.OrderVisited.OrgProcessingResults;
                Kv orgProcessingResults = null;
                if (!string.IsNullOrEmpty(processingResult))
                {
                    var dicData = await _systemDicDataRepository.GetAsync(x => x.DicTypeCode == SysDicTypeConsts.VisitSatisfaction && x.DicDataValue == processingResult, cancellationToken);
                    if (dicData!=null)
                    {
                        orgProcessingResults = new Kv();
                        orgProcessingResults.Key = dicData.DicDataValue;
                        orgProcessingResults.Value = dicData.DicDataName;
                    }
                    if (orgProcessingResults != null)
                    {
                        //主表
                        orderVisit.VisitState = Share.Enums.Order.EVisitState.Visited;
                        orderVisit.VisitTime = notification.OrderVisited.VisitTime;
                        orderVisit.VisitType = notification.OrderVisited.VisitType;
                        orderVisit.IsCanHandle = false;
                        orderVisit.IsCanAiVisit = false;
                        orderVisit.NowEvaluate = orgProcessingResults;
                        await _orderVisitRepository.UpdateAsync(orderVisit, cancellationToken);
                        //子表
                        for (int i = 0; i < orderVisit.OrderVisitDetails.Count; i++)
                        {
                            if (orderVisit.OrderVisitDetails[i].VisitTarget == EVisitTarget.Seat)
                            {
                                orderVisit.OrderVisitDetails[i].VoiceEvaluate = (EVoiceEvaluate)int.Parse(orgProcessingResults.Key);
                                orderVisit.OrderVisitDetails[i].SeatEvaluate = (ESeatEvaluate)int.Parse(orgProcessingResults.Key);
                            }
                            else
                            {
                                orderVisit.OrderVisitDetails[i].OrgProcessingResults = orgProcessingResults;
                            }
                            orderVisit.OrderVisitDetails[i].VisitContent = notification.OrderVisited.VisitContent;
                        }
                        await _orderVisitedDetailRepository.UpdateRangeAsync(orderVisit.OrderVisitDetails, cancellationToken);
                        //工单
                        orderVisit.Order.Visited(orgProcessingResults.Key, orgProcessingResults.Value);
                        await _orderRepository.UpdateAsync(orderVisit.Order);
                    }
                }
            }
        }
    }
}
